
# q-expansion of newform 2880.2.h.d, downloaded from the LMFDB on 07 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 2880
weight = 2
poly_data = [1, 0, 0, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, 0, 1, 0], 1], [[0, 1, 0, 1], 1], [[0, 1, 0, -1], 1]]

hecke_ring_character_values = [[2431, [-1, 0, 0, 0]], [901, [1, 0, 0, 0]], [641, [-1, 0, 0, 0]], [577, [1, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0], [0, 0, 0, 0], [0, -1, 0, 0], [0, 2, -1, 0], [4, 0, 0, 1], [2, 0, 0, -1], [0, -4, -2, 0], [0, 0, 0, 0], [-2, 0, 0, 2], [0, 6, 0, 0], [0, -2, -6, 0], [2, 0, 0, -5], [0, -4, 5, 0], [0, 4, -2, 0], [0, 0, 0, -4], [0, 4, 2, 0], [8, 0, 0, 1], [-10, 0, 0, 2], [0, -8, 0, 0], [0, 0, 0, 4], [-2, 0, 0, 6], [0, 6, -6, 0], [12, 0, 0, 2], [0, 0, -3, 0], [-6, 0, 0, -6], [0, 2, -2, 0], [0, 2, -11, 0], [12, 0, 0, -2], [10, 0, 0, 2], [0, -4, -4, 0], [0, -2, -9, 0], [0, 0, 0, -9], [0, 8, -8, 0], [0, -2, -4, 0], [0, -6, 2, 0], [0, -10, 4, 0], [14, 0, 0, 3], [0, -8, -6, 0], [2, 0, 0, -14], [0, 0, -6, 0], [0, 0, 0, 17], [-10, 0, 0, -6], [12, 0, 0, 6], [2, 0, 0, -12], [0, -10, 8, 0], [0, 10, -4, 0], [0, 26, 0, 0], [0, 10, 9, 0], [0, 0, 0, -4], [-2, 0, 0, -12], [0, 12, 10, 0], [0, 0, 0, 18], [12, 0, 0, -12], [4, 0, 0, -7], [0, 0, 0, 0], [18, 0, 0, 6], [0, -2, 4, 0], [0, -22, 0, 0], [18, 0, 0, 9], [0, 24, 1, 0], [0, 4, 8, 0], [0, -18, 8, 0], [0, -24, 0, 0], [-12, 0, 0, 10], [2, 0, 0, -4], [0, 6, 16, 0], [0, 12, -12, 0], [-18, 0, 0, 6], [-28, 0, 0, -4], [-6, 0, 0, 18], [0, -16, -10, 0], [16, 0, 0, -2], [0, 2, 19, 0], [-2, 0, 0, -9], [0, 2, 8, 0], [-16, 0, 0, -4], [0, -18, -4, 0], [-2, 0, 0, 9], [0, 12, 9, 0], [-10, 0, 0, 20], [-20, 0, 0, 13], [6, 0, 0, 0], [12, 0, 0, 10], [-6, 0, 0, 6], [0, 10, 20, 0], [-4, 0, 0, 16], [0, -20, -1, 0], [-6, 0, 0, -6], [0, 22, -2, 0], [0, -10, 5, 0], [0, 0, 0, -22], [-16, 0, 0, 8], [0, 2, 9, 0], [-24, 0, 0, 5], [0, 4, 16, 0], [-8, 0, 0, 20], [0, 6, 10, 0], [0, -12, -9, 0], [0, 16, 6, 0], [-26, 0, 0, 2], [0, -8, 6, 0], [0, -12, 6, 0], [-12, 0, 0, -4], [0, 12, 15, 0], [0, 12, 12, 0], [-10, 0, 0, -18], [12, 0, 0, -10], [0, 16, 4, 0], [-28, 0, 0, 10], [28, 0, 0, 8], [0, -18, -15, 0], [-18, 0, 0, -15], [0, -24, 2, 0], [0, -6, 12, 0], [0, 10, 2, 0], [0, -28, 11, 0], [0, -4, 22, 0], [-24, 0, 0, -8], [0, 14, -4, 0], [8, 0, 0, 1], [-2, 0, 0, -30], [26, 0, 0, -18], [0, -18, -8, 0], [8, 0, 0, -2], [0, -8, 0, 0], [0, 18, 6, 0], [10, 0, 0, 12], [8, 0, 0, -2], [0, -34, -5, 0], [22, 0, 0, 7], [0, 28, 4, 0], [-48, 0, 0, 0], [0, 26, -2, 0], [26, 0, 0, -19], [0, 4, -19, 0], [-8, 0, 0, -4], [0, 30, -8, 0], [0, -28, 10, 0], [0, 16, -2, 0], [0, 12, -17, 0], [0, 26, 16, 0], [0, -18, 8, 0], [0, -2, 3, 0], [20, 0, 0, -16], [-6, 0, 0, -18], [0, 0, 0, 30], [-26, 0, 0, -3], [0, 16, -14, 0], [0, 26, 0, 0], [26, 0, 0, 14], [-14, 0, 0, -11], [0, 0, 25, 0], [0, 0, 30, 0], [10, 0, 0, -2], [0, -16, -10, 0], [-24, 0, 0, -4], [0, 10, -2, 0], [0, -36, 7, 0], [2, 0, 0, 12], [0, -14, 20, 0], [12, 0, 0, -28], [0, 0, 32, 0], [0, -14, 9, 0], [20, 0, 0, -5], [0, 40, 2, 0], [0, 0, 0, -28], [0, 22, 4, 0], [-18, 0, 0, -9], [16, 0, 0, 16], [0, 0, 14, 0], [-16, 0, 0, -1], [14, 0, 0, -8], [-4, 0, 0, -4], [-10, 0, 0, -32], [0, -10, -18, 0], [0, -8, 13, 0], [0, -46, 8, 0], [0, -42, 2, 0], [0, -18, -3, 0], [-22, 0, 0, 0], [0, -30, -3, 0], [36, 0, 0, 5], [-10, 0, 0, -5], [0, 12, 12, 0], [32, 0, 0, 8], [0, -10, -4, 0], [-10, 0, 0, 21], [0, 40, -4, 0], [0, 0, 0, -36], [24, 0, 0, -14], [-2, 0, 0, -20], [12, 0, 0, -4], [0, -2, -36, 0], [0, -6, 30, 0], [-20, 0, 0, 2], [0, -12, 24, 0], [-26, 0, 0, 28], [22, 0, 0, -7], [0, -12, 20, 0], [32, 0, 0, 20], [0, -42, -16, 0], [0, -2, -6, 0], [26, 0, 0, 19], [16, 0, 0, -28], [12, 0, 0, 17], [0, -14, -28, 0], [0, 2, 18, 0], [-40, 0, 0, 4], [0, 0, 9, 0], [0, -32, -8, 0], [34, 0, 0, 20], [0, 22, 2, 0], [0, 22, 15, 0], [16, 0, 0, 4], [-12, 0, 0, -4], [-6, 0, 0, -12], [0, -18, 21, 0], [0, -12, -3, 0], [40, 0, 0, -4], [0, -12, -30, 0], [30, 0, 0, 12], [0, 22, -24, 0], [0, 20, 29, 0], [0, -22, -21, 0], [40, 0, 0, -22], [-42, 0, 0, -12], [0, 60, -8, 0], [12, 0, 0, -34], [0, -26, -7, 0], [-32, 0, 0, 19], [6, 0, 0, -15], [0, -6, -12, 0], [0, 50, 4, 0], [0, 0, -29, 0], [0, 40, 6, 0], [6, 0, 0, 2], [2, 0, 0, 12], [0, -10, 32, 0], [-36, 0, 0, 5], [36, 0, 0, 16], [-32, 0, 0, 8], [0, -20, -12, 0], [0, -18, 15, 0], [-14, 0, 0, 22], [0, 16, -22, 0], [24, 0, 0, 36], [0, -10, -23, 0], [0, 0, 0, -11], [0, 12, 0, 0], [-18, 0, 0, 30], [-42, 0, 0, -9], [0, 24, 21, 0], [6, 0, 0, 22], [-10, 0, 0, 16], [0, 28, -14, 0], [48, 0, 0, -19], [50, 0, 0, 18], [0, 8, 4, 0], [0, 28, 14, 0], [2, 0, 0, -22], [0, -10, -9, 0], [-16, 0, 0, 16], [-14, 0, 0, 28], [14, 0, 0, -3], [0, -28, 28, 0], [0, -12, 0, 0], [0, 14, 32, 0], [0, 24, -33, 0], [0, 4, -6, 0], [0, 30, 20, 0], [-14, 0, 0, -12], [0, 48, 0, 0], [2, 0, 0, 40], [0, -46, 18, 0], [-2, 0, 0, 28], [0, -2, -9, 0], [-12, 0, 0, 12], [-10, 0, 0, 2], [18, 0, 0, -12], [0, 0, 0, 35], [-42, 0, 0, 14], [0, 10, 28, 0], [-8, 0, 0, 28], [58, 0, 0, -16], [0, 32, -34, 0], [-24, 0, 0, 6], [54, 0, 0, 12], [0, 18, -24, 0], [0, -10, -24, 0], [0, -32, -29, 0], [0, -6, 20, 0], [20, 0, 0, 14], [0, 0, 30, 0], [24, 0, 0, 15], [38, 0, 0, -27], [0, 42, -2, 0], [0, -38, -14, 0], [0, 48, 18, 0], [-36, 0, 0, 1], [0, 4, -6, 0], [-50, 0, 0, -24], [0, -6, 8, 0], [0, 22, 0, 0]]
